// src/mock/mock-client.js
(function() {
  if (window.dvlp) {
    return;
  }
  var RE_WEB_SOCKET_PROTOCOL = /wss?:/;
  var originalXMLHttpRequestOpen = window.XMLHttpRequest.prototype.open;
  var originalFetch = window.fetch;
  var cache = [].map(function(mockData) {
    mockData.originRegex = new RegExp(mockData.originRegex);
    mockData.pathRegex = new RegExp(mockData.pathRegex);
    return mockData;
  });
  var events = cache.reduce(function(events2, mockData) {
    if (mockData.events) {
      events2[mockData.href] = mockData.events;
    }
    return events2;
  }, {});
  var networkDisabled = false;
  var reroute = false;
  window.XMLHttpRequest.prototype.open = function open(method, href) {
    var hrefAndMock = matchHref(href);
    href = hrefAndMock[0];
    var mockData = hrefAndMock[1];
    if (mockData) {
      if (mockData.response) {
        var xhr = this;
        var mockResponse = resolveMockResponse(mockData);
        this.send = function send() {
          if (mockResponse.status === 0) {
            return;
          }
          var body = typeof mockResponse.body === "string" ? mockResponse.body : JSON.stringify(mockResponse.body);
          Object.defineProperties(xhr, {
            readyState: {
              value: 4
            },
            response: {
              get: function() {
                if (mockData.callback) {
                  setTimeout(mockData.callback, 0);
                }
                return body;
              }
            },
            responseText: {
              get: function() {
                return this.response;
              }
            },
            responseURL: {
              value: href
            },
            status: {
              get: function() {
                return mockResponse.status;
              }
            }
          });
          console.log("mocking xhr response (with local data) for: " + parseOriginalHref(href));
          if (xhr.onreadystatechange && typeof xhr.onreadystatechange === "function") {
            xhr.onreadystatechange({ currentTarget: xhr });
          }
          xhr.onload({ currentTarget: xhr });
        };
      } else if (mockData.callback) {
        this.addEventListener("loadend", function() {
          mockData.callback();
        });
      }
    }
    console.log("mocking xhr response (with remote data) for: " + parseOriginalHref(href));
    return originalXMLHttpRequestOpen.call(this, method, href);
  };
  if (typeof Proxy !== "undefined") {
    if (typeof fetch !== "undefined") {
      window.fetch = new Proxy(window.fetch, {
        apply: function(target, ctx, args) {
          var options = args[1] || {};
          var hrefAndMock = matchHref(args[0]);
          var href = hrefAndMock[0];
          var mockData = hrefAndMock[1];
          args[0] = href;
          if (mockData) {
            if (mockData.response) {
              var mockResponse = resolveMockResponse(mockData, options);
              if (mockResponse.status === 0) {
                return new Promise(function() {
                }, function() {
                });
              }
              var body = typeof mockResponse.body === "string" ? mockResponse.body : JSON.stringify(mockResponse.body);
              var res = new Response(body, {
                headers: mockResponse.headers,
                status: mockResponse.status
              });
              if (mockData.callback) {
                setTimeout(mockData.callback, 0);
              }
              console.log("mocking fetch response (with local data) for: " + parseOriginalHref(href));
              return Promise.resolve(res);
            } else if (mockData.callback) {
              return Reflect.apply(target, ctx, args).then(function(response) {
                setTimeout(mockData.callback, 0);
                return response;
              }).catch(function(err) {
                setTimeout(mockData.callback, 0);
                throw err;
              });
            }
          }
          console.log("mocking fetch response (with remote data) for: " + parseOriginalHref(href));
          return Reflect.apply(target, ctx, args);
        }
      });
    }
    if (typeof EventSource !== "undefined") {
      window.EventSource = new Proxy(window.EventSource, {
        construct: function(target, args) {
          if (!args[0].includes("dvlpreload")) {
            args[0] = matchHref(args[0])[0];
            console.log("mocking EventSource response (with remote data) for: " + args[0]);
          }
          return Reflect.construct(target, args);
        }
      });
    }
    if (typeof WebSocket !== "undefined") {
      window.WebSocket = new Proxy(window.WebSocket, {
        construct: function(target, args) {
          args[0] = matchHref(args[0])[0];
          console.log("mocking WebSocket response (with remote data) for: " + args[0]);
          return Reflect.construct(target, args);
        }
      });
    }
  }
  window.dvlp = {
    events,
    cache,
    disableNetwork: function disableNetwork(rerouteAllRequests) {
      networkDisabled = true;
      reroute = rerouteAllRequests || false;
    },
    enableNetwork: function enableNetwork() {
      networkDisabled = false;
      reroute = false;
    },
    mockResponse: function mockResponse(req, res, once, onMockCallback) {
      var ignoreSearch = req && typeof req === "object" && req.url !== void 0 && req.type === void 0 && req.ignoreSearch || false;
      var url = getUrl(req);
      var originRegex = new RegExp(url.origin.replace(/http:|https:/, "https?:").replace("ws:", "wss?:").replace("//", "\\/\\/"));
      var pathRegex = new RegExp(url.pathname.replace(/\//g, "\\/"));
      if (typeof res !== "function") {
        if (res && !res.body) {
          res = { body: res, headers: {} };
        }
      }
      var mock = {
        callback: onMockCallback,
        href: url.href,
        ignoreSearch,
        once: once || false,
        originRegex,
        pathRegex,
        response: res,
        search: url.search
      };
      cache.unshift(mock);
      return function() {
        remove(mock);
      };
    },
    pushEvent: function pushEvent(stream, event) {
      originalFetch("/dvlp/push-event", {
        method: "POST",
        headers: {
          Accept: "application/json",
          "Content-Type": "application/json"
        },
        body: JSON.stringify({ stream, event })
      });
    }
  };
  function matchHref(href) {
    var url = getUrl(href);
    if (url.pathname === "/dvlpreload") {
      return [href];
    }
    var origin = url.origin.indexOf(url.host) === -1 ? url.origin + url.host : url.origin;
    var mockData;
    for (var i = 0; i < cache.length; i++) {
      var mock = cache[i];
      if (!mock.originRegex.test(origin) || !mock.ignoreSearch && mock.search && !isEqualSearch(url.search, mock.search)) {
        continue;
      }
      if (mock.pathRegex.exec(url.pathname) != null) {
        mockData = mock;
        break;
      }
    }
    if (mockData) {
      if (mockData.once) {
        remove(mockData);
      }
      href = (RE_WEB_SOCKET_PROTOCOL.test(url.protocol) ? "ws:" : location.protocol) + "//" + location.host + location.pathname + "?dvlpmock=" + encodeURIComponent(url.href);
    } else if (location.host !== url.host) {
      if (reroute) {
        url.host = location.host;
        href = url.href;
      } else if (networkDisabled) {
        throw Error("network connections disabled. Unable to request " + parseOriginalHref(href));
      }
    }
    return [href, mockData];
  }
  function resolveMockResponse(mockData, requestOptions = {}) {
    var mockResponse = mockData.response;
    var resolved = {
      body: "",
      headers: mockData.response.headers || {},
      status: 0
    };
    if (typeof mockResponse === "function") {
      requestOptions.url = mockData.href;
      mockResponse(requestOptions, {
        end: function end(data) {
          resolved.body = data;
        },
        setHeader: function setHeader(name, value) {
          resolved.headers[name] = value;
        },
        write: function write(chunk) {
          resolved.body += chunk;
        },
        writeHead: function writeHead(statusCode, headers) {
          resolved.status = statusCode;
          if (headers) {
            resolved.headers = headers;
          }
        }
      });
    } else if (mockResponse.error) {
      resolved.body = "error";
      resolved.status = 500;
    } else if (mockResponse.missing) {
      resolved.body = "missing";
      resolved.status = 404;
    } else if (!mockResponse.hang) {
      resolved.body = mockResponse.body;
      resolved.status = mockResponse.status || 200;
    }
    return resolved;
  }
  function getUrl(href) {
    href = typeof href === "string" ? href : href.url;
    try {
      return new URL(href, location.href);
    } catch (err) {
      var a = document.createElement("a");
      a.href = href;
      return {
        protocol: a.protocol,
        host: a.host,
        origin: a.protocol + "//" + a.host,
        pathname: (a.pathname.charAt(0) !== "/" ? "/" : "") + a.pathname,
        search: a.search,
        get href() {
          return this.origin + this.pathaname + this.search;
        }
      };
    }
  }
  function remove(mockData) {
    for (var i = 0; i < cache.length; i++) {
      if (mockData === cache[i]) {
        cache.splice(i, 1);
      }
    }
  }
  function isEqualSearch(search1, search2) {
    var searchMap1 = parseSearch(search1);
    var searchMap2 = parseSearch(search2);
    if (Object.keys(searchMap1).length !== Object.keys(searchMap2).length) {
      return false;
    }
    for (var key in searchMap1) {
      var values1 = searchMap1[key];
      var values2 = searchMap2[key];
      if (!values2 || values1.length !== values2.length) {
        return false;
      }
      for (var i = 0; i < values1.length; i++) {
        var value = values1[i];
        if (values2.indexOf(value) === -1) {
          return false;
        }
      }
    }
    return true;
  }
  function parseSearch(search) {
    search = search.slice(1).split("&");
    var searchMap = {};
    for (var i = 0; i < search.length; i++) {
      var keyVal = search[i].split("=");
      var key = keyVal[0];
      var val = keyVal[1];
      if (!(key in searchMap)) {
        searchMap[key] = [];
      }
      searchMap[key].push(val);
    }
    return searchMap;
  }
  function parseOriginalHref(href) {
    if (href.indexOf("?dvlpmock") === -1) {
      return href;
    }
    return decodeURIComponent(href.split("?dvlpmock=")[1]);
  }
})();

// src/test-browser/index.js
var testBrowser = {
  disableNetwork(rerouteAllRequests) {
    return window.dvlp.disableNetwork(rerouteAllRequests);
  },
  enableNetwork() {
    return window.dvlp.enableNetwork();
  },
  mockResponse(req, res, once, onMockCallback) {
    return window.dvlp.mockResponse(req, res, once, onMockCallback);
  },
  pushEvent(stream, event) {
    return window.dvlp.pushEvent(stream, event);
  }
};
export {
  testBrowser
};
